//#ifdef QT_QML_DEBUG
#include <QtQuick>
//#endif

#include <sailfishapp.h>

#include "login.h"
#include "settings.h"
#include "sync.h"

#include "models/roommodel.h"

int main(int argc, char *argv[]) {
    // SailfishApp::main() will display "qml/harbour-spoon.qml", if you need more
    // control over initialization, you can use:
    //
    //   - SailfishApp::application(int, char *[]) to get the QGuiApplication *
    //   - SailfishApp::createView() to get a new QQuickView * instance
    //   - SailfishApp::pathTo(QString) to get a QUrl to a resource file
    //   - SailfishApp::pathToMainQml() to get a QUrl to the main QML file
    //

    // return SailfishApp::main(argc, argv);

    // To display the view, call "show()" (will show fullscreen on device).
    // Set up QML engine.
    qRegisterMetaType<Room>("Room");
    qRegisterMetaType<QSharedPointer<Room>>("QSharedPointer<Room>");

    QScopedPointer<QGuiApplication> app(SailfishApp::application(argc, argv));
    QScopedPointer<QQuickView> v(SailfishApp::createView());
    Login login;
    v->rootContext()->setContextProperty("login", &login);

    RoomModel rooms;
    Sync sync(rooms, &login);
    QObject::connect(&login, &Login::loginSuccess, &sync, &Sync::startSync);
    QObject::connect(&sync, &Sync::newRoom, &rooms, &RoomModel::addRoom, Qt::QueuedConnection);

    QSharedPointer<Room> a = QSharedPointer<Room>(new Room);
    a->tag = "group";
    a->lastMessage = "Hallo";
    a->unreadNotifications = 5;
    a->name = "RoomA";
    rooms.addRoom(a);
    QSharedPointer<Room> b = QSharedPointer<Room>(new Room);
    b->tag = "group";
    b->lastMessage = "Hallo was ist los mit dir. Papa bist du müde!";
    b->unreadNotifications = 3;
    b->name = "RoomB";
    rooms.addRoom(b);
    QSharedPointer<Room> c = QSharedPointer<Room>(new Room);
    c->tag = "chat";
    c->lastMessage = "Nananana";
    c->unreadNotifications = 0;
    c->name = "RoomC";
    rooms.addRoom(c);
    QSharedPointer<Room> i = QSharedPointer<Room>(new Room);
    i->tag = "invite";
    i->lastMessage = "Hey, Nico hier!";
    i->unreadNotifications = 3;
    i->name = "Example invite";
    rooms.addRoom(i);
    v->rootContext()->setContextProperty("rooms", &rooms);

    // If you wish to publish your app on the Jolla harbour, it is recommended
    // that you prefix your internal namespaces with "harbour.".
    //
    // For details see:
    // https://harbour.jolla.com/faq#1.5.0
    // qmlRegisterType<DemoModel>("com.example", 1, 0, "DemoModel");

    // Start the application.
    v->setSource(SailfishApp::pathTo("qml/harbour-spoon.qml"));
    v->show();

    return app->exec();
}
