#include "roommodel.h"

#include <algorithm>

Q_DECLARE_METATYPE(Room);
Q_DECLARE_METATYPE(QSharedPointer<Room>);

static auto byTag = [](QSharedPointer<Room> a, QSharedPointer<Room> b) { return a->tag < b->tag; };

void RoomModel::addRoom(QSharedPointer<Room> room) {
    auto pos = std::upper_bound(std::begin(joinedRooms), std::end(joinedRooms), room, byTag);
    int posIdx = (int)invites.size() + std::distance(joinedRooms.begin(), pos);
    beginInsertRows(QModelIndex(), posIdx, posIdx);
    joinedRooms.insert(pos, room);
    endInsertRows();
}

void RoomModel::addInvite(QSharedPointer<Room> room) {

    beginInsertRows(QModelIndex(), (int)invites.size(), (int)invites.size());
    invites.push_back(room);
    endInsertRows();
}

QHash<int, QByteArray> RoomModel::roleNames() const {
    QHash<int, QByteArray> roles;
    roles[RoomNameRole] = "roomName";
    roles[UnreadNotificationsRole] = "unreadNotifications";
    roles[LastMessageRole] = "lastMessage";
    roles[RoomPictureRole] = "roomPicture";
    roles[TagRole] = "tag";
    return roles;
}

int RoomModel::rowCount(const QModelIndex &parent) const {
    Q_UNUSED(parent);
    return (int)invites.size() + joinedRooms.size();
}

QVariant RoomModel::data(const QModelIndex &index, int role) const {

    if (index.row() < 0 || index.row() >= rowCount())
        return QVariant();

    const auto &room = [this, index]() {
        if (index.row() < (int)invites.size())
            return invites[index.row()];
        else
            return joinedRooms[index.row() - invites.size()];
    }();

    switch (role) {
    case RoomNameRole:
        return QString::fromStdString(room->name);
    case UnreadNotificationsRole:
        return room->unreadNotifications;
    case LastMessageRole:
        return QString::fromStdString(room->lastMessage);
    case RoomPictureRole:
        return QString::fromStdString(room->picture);
    case TagRole:
        return QString::fromStdString(room->tag);

    default:
        return QVariant();
    }
}

RoomModel::RoomModel(QObject *parent) { Q_UNUSED(parent); }
