#ifndef ROOMMODEL_H
#define ROOMMODEL_H

#include <QAbstractListModel>
#include <QSharedPointer>

#include <map>
#include <string>
#include <vector>

struct Room {
    std::string name, lastMessage, picture, tag;
    int unreadNotifications;
};

class RoomModel : public QAbstractListModel {
    Q_OBJECT
  public:
    enum RoomRoles {
        RoomNameRole,
        UnreadNotificationsRole,
        LastMessageRole,
        RoomPictureRole,
        TagRole,
    };

    QHash<int, QByteArray> roleNames() const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;

    RoomModel(QObject *parent = 0);

  public slots:
    void addRoom(QSharedPointer<Room> room);
    void addInvite(QSharedPointer<Room> room);

  private:
    std::vector<QSharedPointer<Room>> invites;
    std::vector<QSharedPointer<Room>> joinedRooms;
};

#endif // ROOMMODEL_H
