#include "settings.h"

#include <QDebug>

Q_DECLARE_METATYPE(std::string)
Q_DECLARE_METATYPE(uint16_t)

using std::experimental::optional;

Settings::Settings() {}

void Settings::access_token(const std::string &access_token) {
    settings.setValue(QString::fromStdString("access_token"), QString::fromStdString(access_token));
}

optional<std::string> Settings::access_token() {
    if (settings.contains(QStringLiteral("access_token")))
        return settings.value(QStringLiteral("access_token")).value<std::string>();
    else {
        return std::experimental::nullopt;
    }
}

void Settings::device_id(const std::string &s) {
    settings.setValue(QStringLiteral("device_id"), QString::fromStdString(s));
}

optional<std::string> Settings::device_id() {
    if (settings.contains(QStringLiteral("device_id")))
        return settings.value(QStringLiteral("device_id")).value<std::string>();
    else {
        return std::experimental::nullopt;
    }
}

void Settings::server(const std::string &s) { settings.setValue(QStringLiteral("server"), QString::fromStdString(s)); }

optional<std::string> Settings::server() {
    if (settings.contains(QStringLiteral("server")))
        return settings.value(QStringLiteral("server")).value<std::string>();
    else {
        return std::experimental::nullopt;
    }
}

void Settings::user(const mtx::identifiers::User &s) {
    settings.setValue(QStringLiteral("user"), QString::fromStdString(s.to_string()));
}

optional<mtx::identifiers::User> Settings::user() {

    if (settings.contains(QStringLiteral("user"))) {
        std::string s = settings.value(QStringLiteral("user")).value<std::string>();
        try {
            return mtx::identifiers::parse<mtx::identifiers::User>(s);
        } catch (std::exception &e) {
            qWarning() << "Exception getting user from settings: " << QString::fromStdString(e.what())
                       << " while parsing input: '" << QString::fromStdString(s) << "'";
            return std::experimental::nullopt;
        }
    } else {
        return std::experimental::nullopt;
    }
}

void Settings::server_port(uint16_t s) { settings.setValue(QStringLiteral("server_port"), s); }

optional<uint16_t> Settings::server_port() {
    if (settings.contains(QStringLiteral("server_port")))
        return settings.value(QStringLiteral("server_port")).value<uint16_t>();
    else {
        return std::experimental::nullopt;
    }
}

void Settings::delete_access_token() { settings.remove(QStringLiteral("access_token")); }
